#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>	/*	use setmode	*/
#include <io.h>		/*	use setmode	*/
#include "portableio.h"

#include "types.h"
#include "error.h"
#include "wave.h"

#ifdef MFCPCM
  #include <sys/stat.h>
#endif /* MFCPCM */

long	dataSize = 0;			/*	remain samples size(by Jun)	*/

static bool checkString(FILE *file,char *string)
{
    char temp[1024];
    size_t  length = strlen(string);						/*	fixed type different. cast(int to size_t)	*/
    if(fread(temp,1,length,file)!=length) ERROR("Premature EOF in input !?!");
    temp[length] = (char)0;
    return (bool)(strcmp(temp,string) ? false: true);		/*	fixed type different. cast(int to bool)	*/
}

void wave_close()
{
    fclose(config.wave.file);
}

bool wave_open()
{
    static char    *channel_mappings[] = {NULL,"mono","stereo"};
    unsigned short  wFormatTag;
    unsigned long   dAvgBytesPerSec;
    unsigned short  wBlockAlign;
    long            filesize;
    long            header_size;
#ifdef MFCPCM
    extern int pcm_switch;
    extern int pcm_rate;
    extern int pcm_byte_order;
    struct stat sb;
#endif
#ifdef	WAVCH
	/*	seek 'data' chunk	*/
	char			chunkString[5];		/*	chunk buffer	*/
	long			skipSize;			/*	chunk size		*/
	int				i;					/*	loop counter for seek stream	*/
#endif	/*	WAVCH	*/

    if (config.from_stdin) {
        config.wave.file = stdin;
		setmode(fileno(stdin), O_BINARY);	/*	samples are binary stream	*/
    } else {
        if((config.wave.file = fopen(config.infile,"rb")) == NULL) ERROR("Unable to open file");
    }

#ifdef MFCPCM /* dodgy PCM hacked back in */
    if (pcm_switch == 1) {  /* force raw pcm reading */
      stat(config.infile, &sb);
      filesize = sb.st_size;
      printf("Raw PCM, ");
      if (pcm_byte_order == 0) {
        config.wave.type = RAW_PCM_LOHI;
	printf("littleEndian, ");
	}
      else {
	config.wave.type = RAW_PCM_HILO;
	printf("bigEndian, ");
	}
      config.wave.channels = 2;
      if (pcm_rate == 0)
        config.wave.samplerate = 44100;
      else
	config.wave.samplerate = pcm_rate;
      printf("%ldHz ", config.wave.samplerate);
      config.wave.bits=16;

/* PCM specific */
    printf("%dbit, ", config.wave.bits);

    config.wave.total_samples = (filesize)/(2*config.wave.channels);
	dataSize = filesize;		/*	set remain samples size(by Jun)	*/
    config.wave.length = config.wave.total_samples/config.wave.samplerate;
    printf("Length: %2ld:%2ld:%2ld\n", (config.wave.length/3600), 
                                       (config.wave.length/60)%60,
                                       (config.wave.length%60)); 
    return true;

    }
    else {  /* WAV file */
#endif /* MFCPCM */
    if(!checkString(config.wave.file,"RIFF")) ERROR("Input not a MS-RIFF file");
    filesize = Read32BitsLowHigh(config.wave.file); /* complete wave chunk size */
    printf("Microsoft RIFF, ");

    if(!checkString(config.wave.file,"WAVE")) ERROR("Input not WAVE audio");
    printf("WAVE audio, ");

/* WAVE FMT format chunk */
    if(!checkString(config.wave.file,"fmt "))  ERROR("Can't find format chunk");
/* my total header size calculations don't work, so this is bogus... */
    header_size = Read32BitsLowHigh(config.wave.file); /* chunk size */

    wFormatTag = (unsigned short)Read16BitsLowHigh(config.wave.file);				/*	cast(to unsigend short)	*/
    if(wFormatTag!=0x0001) ERROR("Unknown WAVE format");
    config.wave.type = WAVE_RIFF_PCM;

    config.wave.channels   = Read16BitsLowHigh(config.wave.file);
    config.wave.samplerate = Read32BitsLowHigh(config.wave.file);
    dAvgBytesPerSec        = Read32BitsLowHigh(config.wave.file);
    wBlockAlign            = (unsigned short)Read16BitsLowHigh(config.wave.file);	/*	cast(to unsigned short)	*/

/* PCM specific */
    if(config.wave.channels>2) ERROR("More than 2 channels\n");
    printf("PCM, %s %ldHz ", 
           channel_mappings[config.wave.channels], config.wave.samplerate);
    config.wave.bits       = Read16BitsLowHigh(config.wave.file);
    if(config.wave.bits!=16) ERROR("NOT 16 Bit !!!\n");
    printf("%dbit, ", config.wave.bits);

#ifdef WAVCH
	/*	seek 'data' chunk(by Jun)	*/
	for (i = 0; i < header_size - 0x10; i++)	/*	skip over unknouwn chunk	*/
	{
		if (getc(config.wave.file) == EOF)
			ERROR("Can't find data chunk");
	}
	dataSize = 0;
	do
	{
		/*	read chunk	*/
		if (fread(chunkString, sizeof(char), 4, config.wave.file) < 4)
			break;
		if (strncmp(chunkString, "data", 4) == 0)			/*	found 'data' chunk	*/
		{
			dataSize = Read32BitsLowHigh(config.wave.file); /* get wave data size */
			break;
		}
		skipSize = Read32BitsLowHigh(config.wave.file);		/* get chunk size */

		for (i = 0; i < skipSize; i++)	/*	skip over unknouwn chunk	*/
		{
			if (getc(config.wave.file) == EOF)
				ERROR("Can't find data chunk");
		}
	}
	while (1);
	if (dataSize == 0)
		ERROR("Can't find data chunk");

    config.wave.total_samples = dataSize/(2*config.wave.channels);
#else
    if(!checkString(config.wave.file,"data")) ERROR("Can't find data chunk");

    header_size = ftell(config.wave.file);
    fseek(config.wave.file, 0, SEEK_END);
    filesize = ftell(config.wave.file);
    fseek(config.wave.file, header_size, SEEK_SET);

    config.wave.total_samples = (filesize-header_size)/(2*config.wave.channels);
	dataSize = filesize;		/*	set remain samples size(by Jun)	*/
#endif	/*	WAVCH	*/
    config.wave.length = config.wave.total_samples/config.wave.samplerate;
    printf("Length: %2ld:%2ld:%2ld\n", (config.wave.length/3600), 
                                       (config.wave.length/60)%60,
                                       (config.wave.length%60)); 
    return true;
#ifdef MFCPCM
    }  /* ends WAV reading */
#endif /* MFCPCM */
}

static int read_samples(short *sample_buffer,
                        int    frame_size)
{
    int samples_read;

	if (dataSize <= 0)
		return 0;	/*	read samples completed	*/

    switch(config.wave.type)
    {
        default          :
            ERROR("[read_samples], wave filetype not supported");

        case RAW_PCM_LOHI :
            samples_read = fread(sample_buffer,sizeof(short),frame_size, config.wave.file);
    
            /* LOHI PCM Samples are little-endian, */
            /* we must swap if this is a big-endian machine */

           if(config.byte_order==order_bigEndian) 
               SwapBytesInWords(sample_buffer,samples_read);

           if(samples_read<frame_size && samples_read>0) 
           { /* Pad sample with zero's */
               while(samples_read<frame_size) sample_buffer[samples_read++] = 0;
           }

		   dataSize -= samples_read * sizeof(short);	/*	decreas remain samples (by Jun)	*/
		   break;

        case RAW_PCM_HILO :
            samples_read = fread(sample_buffer,sizeof(short),frame_size, config.wave.file);
    
            /* HILO PCM Samples are big-endian, */
            /* we must swap if this is a little-endian machine */

           if(config.byte_order==order_littleEndian) 
               SwapBytesInWords(sample_buffer,samples_read);

           if(samples_read<frame_size && samples_read>0) 
           { /* Pad sample with zero's */
               while(samples_read<frame_size) sample_buffer[samples_read++] = 0;
           }

		   dataSize -= samples_read * sizeof(short);	/*	decreas remain samples (by Jun)	*/
		   break;

        case WAVE_RIFF_PCM :
            samples_read = fread(sample_buffer,sizeof(short),frame_size, config.wave.file);
    
            /* Microsoft PCM Samples are little-endian, */
            /* we must swap if this is a big-endian machine */

           if(config.byte_order==order_bigEndian) 
               SwapBytesInWords(sample_buffer,samples_read);

           if(samples_read<frame_size && samples_read>0) 
           { /* Pad sample with zero's */
               while(samples_read<frame_size) sample_buffer[samples_read++] = 0;
           }

			dataSize -= samples_read * sizeof(short);	/*	decreas remain samples (by Jun)	*/
			if (dataSize <= 0)
			{
				/*	Read samples completed!	*/
				while(dataSize < 0)
				{
					/* Pad sample with zero's */
					sample_buffer[frame_size + dataSize/sizeof(short)] = 0;
					dataSize += sizeof(short);
				}
			}

			break;
    }
    return samples_read;
}


int wave_get(short buffer[2][1152])
/* expects an interleaved 16bit pcm stream from read_samples, which it */
/* de-interleaves into buffer                                          */
{
    static short temp_buf[2304];
    int          samples_read;
    int          j;

    switch(config.mpeg.mode)
    {
        case MODE_MONO  :
            samples_read = read_samples(temp_buf,config.mpeg.samples_per_frame);
            for(j=0;j<1152;j++)
            {
                buffer[0][j] = temp_buf[j];
                buffer[1][j] = 0;
            }
            break;

        case MODE_STEREO : 
        case MODE_JSTEREO:
        case MODE_DUAL   :
            samples_read = read_samples(temp_buf,config.mpeg.samples_per_frame*2);
            for(j=0;j<1152;j++)
            {
                buffer[0][j] = temp_buf[2*j];
                buffer[1][j] = temp_buf[2*j+1];
            }
            break;

        default          :
            ERROR("[wav_get], channel mode not supported");
    }
    return samples_read;
}
 
